@@ -13,6 +13,7 @@ use git2::{AnnotatedCommit, AutotagOption, Blob, Commit, Cred, CredentialType, D
            RepositoryOpenFlags, RepositoryState, ResetType, Revwalk, Signature, Sort, Status, StatusOptions, Time};
 use gpgme::{Context, Protocol};
 use log::{error, info, trace, warn};
+use path_slash::PathBufExt as _;
 use regex::Regex;
 use serde::Deserialize;
 use std::cell::RefCell;
@@ -325,7 +326,7 @@ impl Repo {
       let contents = buf.as_str().ok_or("Buffer was not valid UTF-8")?;
       let out = std::str::from_utf8(&outbuf)?;
 
-      repo.commit_signed(&contents, &out, Some("gpgsig"))?
+      repo.commit_signed(contents, out, Some("gpgsig"))?
     } else {
       repo.commit(head, &sig, &sig, msg, &tree, &[&parent_commit])?
     };
@@ -486,13 +487,21 @@ impl<'r> Slice<'r> {
     let obj = self.object(path)?;
     let tree = obj.into_tree().map_err(|_| bad!("Not a tree: {}", path))?;
     let filter = Regex::new(regex)?;
-    Ok(tree.iter().filter_map(|entry| entry.name().map(|n| n.to_string())).filter(|n| filter.is_match(&n)).collect())
+    Ok(tree.iter().filter_map(|entry| entry.name().map(|n| n.to_string())).filter(|n| filter.is_match(n)).collect())
   }
 
+  #[cfg(not(target_family = "windows"))]
   fn object(&self, path: &str) -> Result<Object> {
     Ok(self.repo.repo()?.revparse_single(&format!("{}:{}", self.refspec.tag(), path))?)
   }
 
+  // Always path issues on windows. See https://github.com/JuliaLang/julia/issues/18724
+  #[cfg(target_family = "windows")]
+  fn object(&self, path: &str) -> Result<Object> {
+    let path = path.replace('\\', "/");
+    Ok(self.repo.repo()?.revparse_single(&format!("{}:{}", self.refspec.tag(), &path))?)
+  }
+
   pub fn date(&self) -> Result<Option<Time>> {
     let obj = match self.repo.repo()?.revparse_single(&format!("{}^{{}}", self.refspec.tag())) {
       Ok(obj) => obj,
@@ -569,7 +578,7 @@ impl<'a> CommitInfo<'a> {
   pub fn summary(&self) -> &str { self.commit.summary().unwrap_or("-") }
   pub fn message(&self) -> &str { self.commit.message().unwrap_or("-") }
   pub fn kind(&self) -> String { extract_kind(self.message()) }
-  pub fn files(&self) -> Result<impl Iterator<Item = String> + 'a> { files_from_commit(&self.repo, &self.commit) }
+  pub fn files(&self) -> Result<impl Iterator<Item = String> + 'a> { files_from_commit(self.repo, &self.commit) }
 
   pub fn buffer(self) -> Result<CommitInfoBuf> {
     Ok(CommitInfoBuf::new(
@@ -916,7 +925,7 @@ fn find_github_info(repo: &Repository, remote_name: &str, auth: &Auth) -> Result
 /// `else_none` is true.
 fn hide_from<'r>(repo: &'r Repository, revwalk: &mut Revwalk<'r>, from: FromTag) -> Result<()> {
   let FromTag { tag, else_none } = from;
-  match repo.revparse_single(&tag) {
+  match repo.revparse_single(tag) {
     Ok(oid) => Ok(revwalk.hide(oid.id())?),
     Err(err) => {
       if !else_none {
@@ -930,7 +939,7 @@ fn hide_from<'r>(repo: &'r Repository, revwalk: &mut Revwalk<'r>, from: FromTag)
 
 fn hide_from_parents<'r>(repo: &'r Repository, revwalk: &mut Revwalk<'r>, from: FromTag) -> Result<()> {
   let FromTag { tag, else_none } = from;
-  match repo.revparse_single(&tag).and_then(|obj| obj.peel_to_commit()) {
+  match repo.revparse_single(tag).and_then(|obj| obj.peel_to_commit()) {
     Ok(commit) => {
       for pid in commit.parent_ids() {
         revwalk.hide(pid)?;
@@ -1034,7 +1043,7 @@ fn files_from_commit<'a>(repo: &'a Repository, commit: &Commit<'a>) -> Result<im
     let ctree = commit.tree()?;
     let diff = repo.diff_tree_to_tree(Some(&ptree), Some(&ctree), Some(&mut DiffOptions::new()))?;
     let iter = DeltaIter::new(diff);
-    Ok(E2::A(iter.map(move |path| path.to_string_lossy().into_owned())))
+    Ok(E2::A(iter.map(move |path| path.to_slash_lossy())))
   } else {
     Ok(E2::B(empty()))
   }
@@ -1141,8 +1150,11 @@ fn verify_current(repo: &Repository) -> Result<()> {
   status_opts.include_ignored(false);
   status_opts.include_untracked(true);
   status_opts.exclude_submodules(false);
-  if repo.statuses(Some(&mut status_opts))?.iter().any(|s| s.status() != Status::CURRENT) {
-    bail!("Repository is not current.");
+
+  let statuses = repo.statuses(Some(&mut status_opts))?;
+  let bad_status = statuses.iter().find(|s| s.status() != Status::CURRENT);
+  if let Some(bad_status) = bad_status {
+    bail!("Repository is not current: {} = {:?}", bad_status.path().unwrap_or("<none>"), bad_status.status());
   }
   Ok(())
 }
